package com.patzn.lims.staff.controller;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.patzn.lims.core.PtConstants;

import com.patzn.lims.core.api.PtResult;
import com.patzn.lims.core.web.ServiceController;
import com.patzn.lims.staff.entity.LmsStaffCertificate;
import com.patzn.lims.staff.service.ILmsStaffCertificateService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 人员证书表 前端控制器
 * </p>
 *
 * @author patzn
 * @since 2018-06-27
 */
@Api(description = "人员证书表")
@RestController
@RequestMapping("/v1/staff_certificate")
public class LmsStaffCertificateController extends ServiceController<ILmsStaffCertificateService,LmsStaffCertificate> {

    @Autowired
    private ILmsStaffCertificateService lmsStaffCertificateService;


    @ApiOperation(value = "分页列表", notes = "分页查询数据列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = PtConstants.PAGE_PAGE, value = "请求数据的页码", required = true, paramType = "query", dataType = "int"),
            @ApiImplicitParam(name = PtConstants.PAGE_ROWS, value = "每页条数", required = true, paramType = "query", dataType = "int"),
            @ApiImplicitParam(name = PtConstants.PAGE_SIDX, value = "排序字段", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = PtConstants.PAGE_SORD, value = "排序方式", paramType = "query", dataType = "String"),
    })
    @PostMapping("/page")
    public PtResult<Page<LmsStaffCertificate>> getPage(LmsStaffCertificate lmsStaffCertificate) {
        return success(lmsStaffCertificateService.page(getPage(), lmsStaffCertificate));
	}


    @ApiOperation(value = "查询列表", notes = "查询数据列表")
    @PostMapping("/list")
    public PtResult<List<LmsStaffCertificate>> getList(LmsStaffCertificate lmsStaffCertificate) {
        return success(lmsStaffCertificateService.list(lmsStaffCertificate));
    }


    @ApiOperation(value = "查询 id 信息", notes = "查询指定 id 的数据内容")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键", required = true, paramType = "path", dataType = "long"),
    })
    @GetMapping("/{id}")
    public PtResult<LmsStaffCertificate> get(@PathVariable("id") Long id) {
        return success(lmsStaffCertificateService.getById(id));
    }


    @ApiOperation(value = "根据 id 修改信息", notes = "根据指定 id 修改该条数据内容")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键", required = true, paramType = "path", dataType = "long"),
    })
    @PutMapping("/{id}")
	public PtResult<Boolean> edit(@PathVariable("id") Long id, LmsStaffCertificate lmsStaffCertificate) {
        lmsStaffCertificate.setId(id);
        return success(lmsStaffCertificateService.updateById(lmsStaffCertificate));
	}


    @ApiOperation(value = "添加", notes = "添加一条数据")
    @PostMapping("/")
	public PtResult<Boolean> add(LmsStaffCertificate lmsStaffCertificate) {
        return success(lmsStaffCertificateService.save(lmsStaffCertificate));
	}


    @ApiOperation(value = "根据 ids 删除", notes = "根据 id 数组删除指定的数据内容")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "主键数组", required = true, paramType = "query", dataType = "array"),
    })
	@DeleteMapping("/")
	public PtResult<Boolean> delete(@RequestParam("ids") Long[] ids) {
        return success(lmsStaffCertificateService.remove(Wrappers.<LmsStaffCertificate>query().in("id", ids)));
	}

}
